==> 2.0.4.tar.gz: ios/Classes/Plugins/Shopping/ShoppingPlugin.swift <==
//
//  ShoppingPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import FireworkVideo
import Flutter
import Foundation

class ShoppingPlugin: NSObject, FlutterPlugin, FireworkVideoShoppingDelegate,
    CartViewControllerProviding {
    var channel: FlutterMethodChannel! = nil

    private var productInfoViewConfigurator: ProductInfoViewConfigurable?

    private var cartIconVisible: Bool?
    private var itemCounts = 0
    private var productInfoViewConfiguration: ProductInfoViewConfiguration?
    private var customClickLinkButtonEnabled = false

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.shoppingModule.rawValue,
            binaryMessenger: registrar.messenger())
        let instance = ShoppingPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case ShoppingPluginMethodName.`init`.rawValue:
            initialize()
            result(nil)
        case ShoppingPluginMethodName.setCartIconVisible.rawValue:
            if let visible = call.arguments as? Bool {
                setCartIconVisible(visible)
            }
            result(nil)
        case ShoppingPluginMethodName.setCartItemCount.rawValue:
            if let count = call.arguments as? Int {
                setCartItemCount(count)
            }
            result(nil)
        case ShoppingPluginMethodName.setProductInfoViewConfiguration.rawValue:
            setProductInfoViewConfiguration(call.arguments as? [String: Any])
            result(nil)
        case ShoppingPluginMethodName.setCustomClickLinkButtonEnabled.rawValue:
            if let enabled = call.arguments as? Bool {
                setCustomClickLinkButtonEnabled(enabled)
            }
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    func initialize() {
        FireworkVideoSDK.shopping.shoppingDelegate = self
        FireworkVideoSDK.shopping.cartViewControllerProvider = self
    }

    func setCartIconVisible(_ visible: Bool) {
        cartIconVisible = visible

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !visible
    }

    func setCartItemCount(_ itemCounts: Int) {
        self.itemCounts = itemCounts

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
    }

    func setProductInfoViewConfiguration(_ config: [String: Any]?) {
        let config = Convert.buildProductInfoViewConfiguration(config)
        guard let rConfig = config else {
            return
        }

        productInfoViewConfiguration = rConfig

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        Self.hydrateProductViewConfig(rConfig, rProductInfoViewConfigurator)
    }

    func setCustomClickLinkButtonEnabled(_ enabled: Bool) {
        customClickLinkButtonEnabled = enabled
    }

    // MARK: - FireworkVideoShoppingDelegate
    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        willDisplayProductInfo productInfoViewConfigurator: ProductInfoViewConfigurable,
        forVideo video: VideoDetails
    ) {
        self.productInfoViewConfigurator = productInfoViewConfigurator
        if let cartIconVisible = self.cartIconVisible {
            productInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !cartIconVisible
        }
        productInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
        if let productInfoViewConfiguration = productInfoViewConfiguration {
            Self.hydrateProductViewConfig(productInfoViewConfiguration, productInfoViewConfigurator)
        }
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping, updateDetailsForProducts products: [ProductID],
        forVideo video: VideoDetails, _ productHydrator: ProductHydrating
    ) {
        channel.invokeMethod(
            ShoppingEventName.updateProductDetails.rawValue, arguments: ["productIds": products]
        ) { (result: Any?) -> Void in
            let products = Convert.buildProducts(result as? [AnyObject])
            guard let rProducts = products else {
                return
            }

            for product in rProducts {
                productHydrator.hydrateProduct(
                    product.productId, { build in
                        return Self.hydrateProduct(product, build)
                    })
            }
        }
    }

    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping,
                          productVariantCTASelected item: SelectedProductVariant,
                          fromVideo video: VideoDetails,
                          ctaCompletionHandler: @escaping ShoppingCTAHandler
    ) {
        channel.invokeMethod(
            ShoppingEventName.ctaButtonClick.rawValue,
            arguments: [
                "url": item.url?.absoluteString ?? "",
                "productId": item.productID,
                "unitId": item.unitID
            ]
        ) { (result: Any?) -> Void in
            guard let result = result as? [String: Any] else {
                return
            }

            guard let ctaResult = Convert.buildShoppingCTAResult(result) else {
                return
            }

            if let res = ctaResult.res,
               let tips = ctaResult.tips,
               tips.count > 0 {
                switch res {
                case .success:
                    ctaCompletionHandler(.feedbackOnly(.success(message: tips)))
                case .fail:
                    ctaCompletionHandler(.feedbackOnly(.failure(message: tips)))
                }
            } else {
                ctaCompletionHandler(.none)
            }
        }
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapLinkButtonAt item: SelectedProductVariant,
        withURL itemURL: String
    ) -> Bool {
        if customClickLinkButtonEnabled {
            channel.invokeMethod(ShoppingEventName.customLinkButtonClick.rawValue, arguments: [
                "url": itemURL,
                "productId": item.productID,
                "unitId": item.unitID
            ])
        }

        return customClickLinkButtonEnabled
    }

    // MARK: - CartViewControllerProviding
    func cartViewController(for video: VideoDetails) -> FireworkVideo.CartViewController {
        channel.invokeMethod(ShoppingEventName.clickCartIcon.rawValue, arguments: [:])
        return CartViewController()
    }
}

extension ShoppingPlugin {
    fileprivate static func hydrateProductViewConfig(
        _ config: ProductInfoViewConfiguration,
        _ productInfoViewConfigurator: ProductInfoViewConfigurable
    ) {
        var productDetailsConfiguration = productInfoViewConfigurator.productDetailsConfiguration

        if let ctaButtonConfiguration = config.ctaButton {
            if let text = ctaButtonConfiguration.text {
                switch text {
                case .addToCart:
                    productDetailsConfiguration.ctaButton.text = .addToCart
                case .shopNow:
                    productDetailsConfiguration.ctaButton.text = .shopNow
                }
            }
            if let backgroundColor = ctaButtonConfiguration.backgroundColor,
                backgroundColor.count > 0 {
                productDetailsConfiguration.ctaButton.buttonConfiguration.backgroundColor = backgroundColor.uicolor()
            }

            if let textColor = ctaButtonConfiguration.textColor,
                textColor.count > 0 {
                productDetailsConfiguration.ctaButton.buttonConfiguration.textColor = textColor.uicolor()
            }

            if let fontSize = ctaButtonConfiguration.fontSize {
                let iOSFontInfo = ctaButtonConfiguration.iOSFontInfo ?? FontInfo()
                productDetailsConfiguration.ctaButton.buttonConfiguration.font = iOSFontInfo.getFont(fontSize)
            }
        }

        if let linkButtonConfiguration = config.linkButton {
            if let isHidden = linkButtonConfiguration.isHidden {
                productDetailsConfiguration.linkButtonConfiguration.isHidden = isHidden
            }
        }

        productInfoViewConfigurator.productDetailsConfiguration = productDetailsConfiguration
    }
}

extension ShoppingPlugin {
    fileprivate static func hydrateProduct(_ product: Product, _ pbuild: ProductBuilder)
        -> ProductBuilder {
            if let name = product.name {
                pbuild.name(name)
            }
            if let description = product.description {
                pbuild.description(description)
            }
            if let isAvailable = product.isAvailable {
                pbuild.isAvailable(isAvailable)
            }
            if let variants = product.units {
                for variant in variants {
                    pbuild.variant(variant.unitId) { pvBuild in
                        return Self.buildVariant(variant, pvBuild)
                    }
                }
            }
        return pbuild
    }

    fileprivate static func buildVariant(
        _ variant: Product.ProductUnit, _ pvBuild: ProductVariantBuilder
    ) -> ProductVariantBuilder {
        if let name = variant.name {
            pvBuild.name(name)
        }
        if let url = variant.url {
            pvBuild.url(url)
        }
        if let imageUrl = variant.imageUrl {
            pvBuild.imageUrl(imageUrl)
        }
        if let price = variant.price {
            if let amount = price.amount, let currencyCode = price.currencyCode {
                pvBuild.formattedPrice(Decimal(amount), currencyCode: currencyCode)
            }
        }
        if let isAvailable = variant.isAvailable {
            pvBuild.isAvailable(isAvailable)
        }
        let options = Self.buildVariantOption(variant.options)
        if let options = options {
            pvBuild.options(options)
        }
        return pvBuild
    }

    fileprivate static func buildVariantOption(_ options: [Product.VariantOption]?) -> [String: String]? {
        guard let options = options else {
            return nil
        }
        var result: [String: String] = [:]
        for option in options {
            guard let name = option.name, let value = option.value else {
                continue
            }
            result[name] = value
        }
        return result
    }
}

==> 2.1.0.tar.gz: ios/Classes/Plugins/Shopping/ShoppingPlugin.swift <==
//
//  ShoppingPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import FireworkVideo
import Flutter
import Foundation

class ShoppingPlugin: NSObject, FlutterPlugin, FireworkVideoShoppingDelegate {
    var channel: FlutterMethodChannel! = nil

    private var productInfoViewConfigurator: ProductInfoViewConfigurable?

    private var cartIconVisible: Bool?
    private var itemCounts = 0
    private var productInfoViewConfiguration: ProductInfoViewConfiguration?
    private var customClickLinkButtonEnabled = false

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.shoppingModule.rawValue,
            binaryMessenger: registrar.messenger())
        let instance = ShoppingPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case ShoppingPluginMethodName.`init`.rawValue:
            initialize()
            result(nil)
        case ShoppingPluginMethodName.setCartIconVisible.rawValue:
            if let visible = call.arguments as? Bool {
                setCartIconVisible(visible)
            }
            result(nil)
        case ShoppingPluginMethodName.setCartItemCount.rawValue:
            if let count = call.arguments as? Int {
                setCartItemCount(count)
            }
            result(nil)
        case ShoppingPluginMethodName.setProductInfoViewConfiguration.rawValue:
            setProductInfoViewConfiguration(call.arguments as? [String: Any])
            result(nil)
        case ShoppingPluginMethodName.setCustomClickLinkButtonEnabled.rawValue:
            if let enabled = call.arguments as? Bool {
                setCustomClickLinkButtonEnabled(enabled)
            }
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    func initialize() {
        FireworkVideoSDK.shopping.shoppingDelegate = self
        FireworkVideoSDK.shopping.cartAction = .custom
    }

    func setCartIconVisible(_ visible: Bool) {
        cartIconVisible = visible

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !visible
    }

    func setCartItemCount(_ itemCounts: Int) {
        self.itemCounts = itemCounts

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
    }

    func setProductInfoViewConfiguration(_ config: [String: Any]?) {
        let config = Convert.buildProductInfoViewConfiguration(config)
        guard let rConfig = config else {
            return
        }

        productInfoViewConfiguration = rConfig

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        Self.hydrateProductViewConfig(rConfig, rProductInfoViewConfigurator)
    }

    func setCustomClickLinkButtonEnabled(_ enabled: Bool) {
        customClickLinkButtonEnabled = enabled
    }

    // MARK: - FireworkVideoShoppingDelegate
    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        willDisplayProductInfo productInfoViewConfigurator: ProductInfoViewConfigurable,
        forVideo video: VideoDetails
    ) {
        self.productInfoViewConfigurator = productInfoViewConfigurator
        if let cartIconVisible = self.cartIconVisible {
            productInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !cartIconVisible
        }
        productInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
        if let productInfoViewConfiguration = productInfoViewConfiguration {
            Self.hydrateProductViewConfig(productInfoViewConfiguration, productInfoViewConfigurator)
        }
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping, updateDetailsForProducts products: [ProductID],
        forVideo video: VideoDetails, _ productHydrator: ProductHydrating
    ) {
        channel.invokeMethod(
            ShoppingEventName.updateProductDetails.rawValue, arguments: ["productIds": products]
        ) { (result: Any?) -> Void in
            let products = Convert.buildProducts(result as? [AnyObject])
            guard let rProducts = products else {
                return
            }

            for product in rProducts {
                productHydrator.hydrateProduct(
                    product.productId, { build in
                        return Self.hydrateProduct(product, build)
                    })
            }
        }
    }

    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping,
                          productVariantCTASelected item: SelectedProductVariant,
                          fromVideo video: VideoDetails,
                          ctaCompletionHandler: @escaping ShoppingCTAHandler
    ) {
        channel.invokeMethod(
            ShoppingEventName.ctaButtonClick.rawValue,
            arguments: [
                "url": item.url?.absoluteString ?? "",
                "productId": item.productID,
                "unitId": item.unitID
            ]
        ) { (result: Any?) -> Void in
            guard let result = result as? [String: Any] else {
                return
            }

            guard let ctaResult = Convert.buildShoppingCTAResult(result) else {
                return
            }

            if let res = ctaResult.res,
               let tips = ctaResult.tips,
               tips.count > 0 {
                switch res {
                case .success:
                    ctaCompletionHandler(.feedbackOnly(.success(message: tips)))
                case .fail:
                    ctaCompletionHandler(.feedbackOnly(.failure(message: tips)))
                }
            } else {
                ctaCompletionHandler(.none)
            }
        }
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapLinkButtonAt item: SelectedProductVariant,
        withURL itemURL: String
    ) -> Bool {
        if customClickLinkButtonEnabled {
            channel.invokeMethod(ShoppingEventName.customLinkButtonClick.rawValue, arguments: [
                "url": itemURL,
                "productId": item.productID,
                "unitId": item.unitID
            ])
        }

        return customClickLinkButtonEnabled
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapCartIconForVideo videoDetails: VideoDetails
    ) {
        channel.invokeMethod(ShoppingEventName.clickCartIcon.rawValue, arguments: [:])
    }
}

extension ShoppingPlugin {
    fileprivate static func hydrateProductViewConfig(
        _ config: ProductInfoViewConfiguration,
        _ productInfoViewConfigurator: ProductInfoViewConfigurable
    ) {
        var productDetailsConfiguration = productInfoViewConfigurator.productDetailsConfiguration

        if let ctaButtonConfiguration = config.ctaButton {
            if let text = ctaButtonConfiguration.text {
                switch text {
                case .addToCart:
                    productDetailsConfiguration.ctaButton.text = .addToCart
                case .shopNow:
                    productDetailsConfiguration.ctaButton.text = .shopNow
                }
            }
            if let backgroundColor = ctaButtonConfiguration.backgroundColor,
                backgroundColor.count > 0 {
                productDetailsConfiguration.ctaButton.buttonConfiguration.backgroundColor = backgroundColor.uicolor()
            }

            if let textColor = ctaButtonConfiguration.textColor,
                textColor.count > 0 {
                productDetailsConfiguration.ctaButton.buttonConfiguration.textColor = textColor.uicolor()
            }

            if let fontSize = ctaButtonConfiguration.fontSize {
                let iOSFontInfo = ctaButtonConfiguration.iOSFontInfo ?? FontInfo()
                productDetailsConfiguration.ctaButton.buttonConfiguration.font = iOSFontInfo.getFont(fontSize)
            }
        }

        if let linkButtonConfiguration = config.linkButton {
            if let isHidden = linkButtonConfiguration.isHidden {
                productDetailsConfiguration.linkButtonConfiguration.isHidden = isHidden
            }
        }

        productInfoViewConfigurator.productDetailsConfiguration = productDetailsConfiguration
    }
}

extension ShoppingPlugin {
    fileprivate static func hydrateProduct(_ product: Product, _ pbuild: ProductBuilder)
        -> ProductBuilder {
            if let name = product.name {
                pbuild.name(name)
            }
            if let description = product.description {
                pbuild.description(description)
            }
            if let isAvailable = product.isAvailable {
                pbuild.isAvailable(isAvailable)
            }
            if let variants = product.units {
                for variant in variants {
                    pbuild.variant(variant.unitId) { pvBuild in
                        return Self.buildVariant(variant, pvBuild)
                    }
                }
            }
        return pbuild
    }

    fileprivate static func buildVariant(
        _ variant: Product.ProductUnit, _ pvBuild: ProductVariantBuilder
    ) -> ProductVariantBuilder {
        if let name = variant.name {
            pvBuild.name(name)
        }
        if let url = variant.url {
            pvBuild.url(url)
        }
        if let imageUrl = variant.imageUrl {
            pvBuild.imageUrl(imageUrl)
        }
        if let price = variant.price {
            if let amount = price.amount, let currencyCode = price.currencyCode {
                pvBuild.formattedPrice(Decimal(amount), currencyCode: currencyCode)
            }
        }
        if let isAvailable = variant.isAvailable {
            pvBuild.isAvailable(isAvailable)
        }
        let options = Self.buildVariantOption(variant.options)
        if let options = options {
            pvBuild.options(options)
        }
        return pvBuild
    }

    fileprivate static func buildVariantOption(_ options: [Product.VariantOption]?) -> [String: String]? {
        guard let options = options else {
            return nil
        }
        var result: [String: String] = [:]
        for option in options {
            guard let name = option.name, let value = option.value else {
                continue
            }
            result[name] = value
        }
        return result
    }
}
